"""This module provides timing utilities including classes and functions for measuring and recording execution time."""

import lazy_loader as lazy

__getattr__, __dir__, __all__ = lazy.attach_stub(__name__, __file__)
